let UIManager = function(eventAggregator, templateManager) {

    /**
     * @type {Array<String|Function>}
     */
    let stateStack = [];

    let isListenerAttached = false;
    let activeWidget;
    let sf = this;
    /**
     * Called every time a container is open
     */
    this.init = () => {
        stateStack = [];
        activeWidget = this.activeWidget = templateManager.container.getActiveWidget();
    };

    if (!isListenerAttached) {
        eventAggregator.subscribe('onContainerOpen', this.init);
        isListenerAttached = true;
    }

    /**
     * Displays the given widget
     * @param {ZhWidgetOjbect} widgetObject
     * @param {boolean} [shouldReload] 
     */
    this.goToWidget = (widgetObject, shouldReload = false) => {
        if (widgetObject && widgetObject.name !== activeWidget.name) {
            widgetObject.activate();
            stateStack.push(activeWidget.name);
            activeWidget.isVisible = false;
            widgetObject.isVisible = true;
            activeWidget = widgetObject;
            if (activeWidget.container.steps) {
                templateManager.container.highlightStep(widgetToOpen.name);
                sf.moveSteps(widgetToOpen);
            }
            if (shouldReload) {
                return widgetObject.reload();
            }
        }
    };

    /**
     * Tryes to find a widget by it's name and display it
     * @param {string} widgetName 
     * @param {boolean} shouldReload 
     */
    this.goToWidgetByName = (widgetName, shouldReload = false, shouldCarryCurRow = false) => {
        let widgetToOpen = templateManager.container.getWidget(widgetName);
        if (widgetToOpen && widgetToOpen.name !== activeWidget.name) {
            widgetToOpen.activate();
            stateStack.push(activeWidget.name);
            activeWidget.isVisible = false;
            widgetToOpen.isVisible = true;
            if (shouldCarryCurRow) {
                widgetToOpen.currentRow = activeWidget.currentRow;
            }
            activeWidget = widgetToOpen;
            if (activeWidget.container.steps) {
                templateManager.container.highlightStep(widgetToOpen.name);
                sf.moveSteps(widgetToOpen);
            }

            if (shouldReload) {
                return widgetToOpen.reload();
            }
        } else {
            console.log(`%can attempt to open the widget: ${widgetName} was made. But this widget wasn't found`, "font-size: 15px");
        }
    };

    /**
     * Displays a widget that is already in the stack 
     * and removes all others in the path
     * 
     * homeScreen -> list -> listItem -> listItemDetails
     * 
     * goBackToStackedWidget('list')
     * 
     * homeScreen
     *  
     * @param {string} widgetName 
     */
    this.goBackToStackedWidget = widgetName => {
        let widgetToGoBack = templateManager.container.getWidget(widgetName);
        let widgetNameIndex = stateStack.indexOf(widgetName);
        if (widgetToGoBack && widgetNameIndex !== -1 && stateStack.length > 0) {
            stateStack = stateStack.slice(0, widgetNameIndex);
            widgetToGoBack.activate();
            activeWidget.isVisible = false;
            widgetToGoBack.isVisible = true;
            activeWidget = widgetToGoBack;
        }
    };

    /**
     * Displays the last widget in the stack
     * or triggers the last callback state in the stack 
     */
    this.goBack = () => {
        if (stateStack.length > 0) {
            if (typeof stateStack[stateStack.length - 1] === 'string') {
                let widgetToGoBackName = stateStack.pop();
                let widgetToGoBack = templateManager.container.getWidget(widgetToGoBackName);
                widgetToGoBack.activate();
                widgetToGoBack.isVisible = true;
                activeWidget.isVisible = false;

                let steps = widgetToGoBack.container.steps;
                if (steps && steps.length > 0) {
                    let footerPositioner = document.getElementsByClassName("footer-positioner")[0];
                    let stepElements = document.getElementsByClassName("footer-wizard-step");

                    steps.find(step => step.active === true).active = false;
                    let newActiveStep = steps.find(step => step.name === widgetToGoBackName);
                    let indexOfStep = steps.indexOf(newActiveStep);
                    newActiveStep.active = true;

                    let curStep = stepElements[indexOfStep];
                    let curStepWidth = parseInt(window.getComputedStyle(curStep, null).getPropertyValue("width"));
                    footerPositioner.scrollLeft = curStep.offsetLeft - (curStepWidth / 3);
                }
                activeWidget = widgetToGoBack;
            } else if (typeof stateStack[stateStack.length - 1] === 'function') {
                (stateStack.pop())();
            }
        }
    };

    /**
     * hides the editor of the active widget, (if this widget is editable)
     */
    this.hideEdit = () => {
        if (activeWidget.hasOwnProperty('isEditing')) {
            this.allowScroll();
            activeWidget.isEditing = false;
            // activeWidget.restoreHeaderActions();
        }
    };

    this.preventScroll = () => document.body.classList.add('no-scroll');

    this.allowScroll = () => document.body.classList.remove('no-scroll');

    this.moveSteps = (widget) => {
        let steps = widget.container.steps;
        if (steps) {
            let footerPositioner = document.getElementsByClassName("footer-positioner")[0];
            let stepElements = document.getElementsByClassName("footer-wizard-step");

            let curStep = stepElements[steps.indexOf(steps.find(step => step.active === true))];
            let curStepWidth = parseInt(window.getComputedStyle(curStep, null).getPropertyValue("width"));
            footerPositioner.scrollLeft = curStep.offsetLeft - (curStepWidth / 3);
        }
    };

    /**
     * @param {Function} callback
     */
    this.pushState = (callback) => stateStack.push(callback);

};

Configuration(function(ContextRegister) {
    ContextRegister.register('UIManager', UIManager);
});
