function ZhFilterPreferences(ScreenService, templateManager, ZhFilterBuilder, ZhUtil, RepositoryFactory, Query, eventEngine, ZHPromise, ZhLogger, ApplicationContext, zhCustomizationsService, filterWidgetsCreator, eventAggregator, preferencesUserService, DecodeValueFieldService) {

	var self = this;
	var gridWidget, filterDescription = [];

	eventAggregator.subscribe("OnListStartUp", getAndApplyDefaultPreference);

	function getAndApplyDefaultPreference(eventData) {
		if (eventData.widget) {
			var userRegistered = preferencesUserService.getPreferenceUserRegistered() || "default";
			self.getDefaultPreferenceByUser(eventData.widget, userRegistered).then(function(defaultPreference) {
				if (defaultPreference) {
					self.applyFilter(defaultPreference, eventData.widget);
				} else {
					eventData.widget.customizationsApplied = eventData.widget.customizationsApplied || {};
					eventData.widget.customizationsApplied.filter = "default";
				}
			});
		}
	}

	this.getDefaultPreferenceByUser = function(widget, userId) {
		return zhCustomizationsService.getWidgetCustomizations(widget, "filter").then(function(preferences) {
			return preferences.filter(function(preference) {
				return preference.mapUserIdDefault && preference.mapUserIdDefault[userId];
			}).shift();
		});
	};

	this.startFilterConfig = function startFiltering(widget) {
		gridWidget = widget;
		gridWidget.__filterConfig = filterWidgetsCreator.createFiltersWidget(gridWidget, functionsFilterListWidget);
		fillFilterList(gridWidget.__filterConfig).then(function() {
			ScreenService.openPopup(gridWidget.__filterConfig);
		});
	};

	this.restoreDefault = function(widget) {
		this.restoreDefaultFilter(widget);
	};

	function getFilterWidget(widget) {
		if (widget.__filterPopup === undefined) {
			widget.__filterPopup = filterWidgetsCreator.createCurrentFormFilterWidget(widget, functionsFormCurrentFilterWidget);
			widget.__filterPopup.showAsModal = false;
			widget.__filterPopup.label = ScreenService.i18n("Filter");
			widget.__filterPopup.fields = prepareFieldstoFormFilter(factoryFields(widget.__filterPopup));
			var searchFields = widget.hideSearchFieldsOnFilter ? [] : prepareFieldstoFormFilter(factorySearchFields(widget.__filterPopup));
			widget.__filterPopup.fields = widget.__filterPopup.fields.concat(searchFields);
			widget.__filterPopup.applyDefaultValue();
			var filters = widget.filterApplied;
			if (filters && filters.length || widget._filterRow) {
				widget.__filterPopup.currentRow = buildCurrentRowByFilters(filters, widget.__filterPopup, widget._filterRow);
			}
		}

		return widget.__filterPopup;
	}

	this.startFiltering = function(widget) {
		gridWidget = widget;
		ScreenService.openPopup(getFilterWidget(gridWidget));
	};

	var filterVisibleFields = function(fields) {
		return fields.filter(function(field) {
			return field.isVisible;
		});
	};

	function currentFilterClickPopup(widget) {
		if (widget.isValid()) {
			var currentRow = widget.currentRow;
			var fields = filterVisibleFields(gridWidget.__filterPopup.fields);
			var filter = {
				_filterRow: ZhUtil.clearRow(currentRow),
				filters: ZhFilterBuilder.buildFilter(fields, currentRow),
				_id: "edited"
			};
			self.applyFilter(filter, gridWidget);
			ScreenService.closePopup();
		}
	}

	function fillFilterList() {
		return zhCustomizationsService.getWidgetCustomizations(gridWidget, "filter").then(function(filters) {
			filters.forEach(function(filter) {
				filter.filterTypeLabel = zhCustomizationsService.getLabelRepositoryType(filter.customizationRepository);
			});
			gridWidget.__filterConfig.dataSource.data = filters;
			ScreenService.updateTemplate();
		});
	}

	this.restoreDefaultFilter = function(widget) {
		var editingWidget = widget || gridWidget;
		var filter = {
			"_id": "default",
			filters: this.getDefaultFilterProperties(editingWidget)
		};
		this.applyFilter(filter, editingWidget);
		this.updateFilterInfo(editingWidget, []);
	};

	function applyFilterEventCallback(eventData) {
		var filter = eventData.e.customization;
		var editingWidget = eventData.e.editingWidget;
		self.applyFilter(filter, editingWidget);
	}

	var fireBeforeBuildFilterEvent = function(editingWidget, filter) {
		var event = editingWidget.beforeGridFilterApply && editingWidget.beforeGridFilterApply(filter);
		return ZHPromise.when(event);
	};

	function _getFilterFields(filters) {
		return filters.filter(function(filter) {
			return !~filter.name.indexOf("SEARCHLIST_");
		});
	}

	function _getSearchFields(filters) {
		var searchFields = Util.clone(filters);
		searchFields = searchFields.filter(function(filter) {
			return ~filter.name.indexOf("SEARCHLIST_");
		});
		searchFields.map(function(filter) {
			filter.name = filter.name.replace("SEARCHLIST_","");
			filter.static = true;
		});
		return searchFields;
	}

	function _applySearchCriteria(searchFields, editingWidget) {
		if (editingWidget.setSearchCriteria) {
			editingWidget.setSearchCriteria(searchFields);
		}
	}

	this.applyFilter = function(filter, editingWidget, callback) {
		editingWidget._filterRow = filter._filterRow;
		var originalFilter = Util.clone(filter.filters);
		fireBeforeBuildFilterEvent(editingWidget, filter).then(function(event) {
			if (!event || !event.isDefaultPrevented()) {
				if (!editingWidget.customizationsApplied) {
					editingWidget.customizationsApplied = {};
				}
				createFilterDescription(editingWidget, originalFilter);
				editingWidget.filterApplied = originalFilter;
				editingWidget.customizationsApplied.filter = filter._id;

				var filterFields;
				var searchFields;
				if (filter.filters) {
					filterFields = _getFilterFields(filter.filters);
					searchFields = _getSearchFields(filter.filters);
				}

				if (filterFields && filterFields.length > 0) {
					filterFields.forEach(function(filter) {
						filter.static = true;
					});
				}
				if (editingWidget.dataSourceFilter) {
					editingWidget.dataSourceFilter = self.getStaticDataSourceFilters(editingWidget.dataSourceFilter);
					if (filterFields && filterFields.length > 0) {
						editingWidget.dataSourceFilter = editingWidget.dataSourceFilter.concat(filterFields);
					}
				}

				_applySearchCriteria(searchFields, editingWidget);

				if (editingWidget.afterGridFilterApply) {
					editingWidget.afterGridFilterApply();
				}

				editingWidget.reload(ZhUtil.getLikeAllFilter(editingWidget, editingWidget.searchCriteria)).then(function() {
					if (originalFilter) {
						self.updateFilterInfo(editingWidget, originalFilter);
					}
					if (callback) {
						callback();
					}
				});
			}
		});
	};

	this.preferencesListOnEnter = function(widgetPreferenceList) {
		var userRegistered = preferencesUserService.getPreferenceUserRegistered() || "default";
		return zhCustomizationsService.getWidgetCustomizations(gridWidget, "filter").then(function(preferences) {
			preferences.forEach(function(preference) {
				preference.defaultPreference = preference.mapUserIdDefault[userRegistered] || false;
				preference.preferenceTypeLabel = zhCustomizationsService.getLabelRepositoryType(preference.customizationRepository);
			});
			widgetPreferenceList.dataSource.data = preferences;
		});
	};

	eventEngine.bindEvent(zhCustomizationsService, "FilterPreferenceOnClick", applyFilterEventCallback);

	var handleSelectMultipleDescription = function(field, currentRow) {
		if (field.template == "field/select-multiple.html" && field.dataSource.data.length > 0) {
			if (currentRow && currentRow[field.name]) {
				field.value(currentRow[field.name]);
			}
			if (field && Util.isArray(field.value())) {
				registers = field.dataSource.data.filter(function(register) {
					return ~field.value().indexOf(register[field.valueField]);
				});
			}
		} else {
			if (field.value() && Util.isArray(field.value()) && field.dataSource.data.length > 0) {
				registers = field.dataSource.data.filter(function(register) {
					return field.value() && register[field.valueField] == field.value();
				}).shift();
				if (registers && Util.isArray(registers)) {
					var teste = [];
					registers.forEach(function(register) {
						teste.push(register[field.descriptionField]);
					});
					return teste.join(", ") || currentRow[field.descriptionField];
				}
			}
		}
	};

	var oldDescriptionCreator = function(field, currentRow) {
		var labelDescription;
		if (field.descriptionField && field.valueField) {
			labelDescription = handleSelectMultipleDescription(field, currentRow);
		} else {
			if (field.outData) {
				labelDescription = currentRow[field.outData[field.valueField]];
			} else {
				labelDescription = currentRow[field.name];
			}
		}
		if (Util.isArray(labelDescription)) {
			labelDescription = labelDescription.map(function(value) {
				if (Util.isString(value)) {
					return value.replace(/^%|%$/g, "");
				}
				return value;
			}).join(" ");
		} else {
			if (Util.isString(labelDescription)) {
				labelDescription = labelDescription.replace(/^%|%$/g, "");
			}
		}
		return labelDescription;
	};

	var createFilterDescription = function(gridWidget, filters) {
		var fields = gridWidget.fields;
		var currentRow = buildCurrentRowByFilters(filters, gridWidget);
		filterDescription = [];
		if (!Util.isEmptyObject(currentRow)) {
			filterVisibleFields(fields).forEach(function(field) {
				var _field = Util.clone(field, ['widget', 'element', 'owner', 'selectWidget', 'intervalWidget']);
				_field.widget = field.widget;
				_field.template = (_field.filterProperties && _field.filterProperties.template) || _field.template;
				var labelDescription = DecodeValueFieldService.model(_field, currentRow[_field.name]) || oldDescriptionCreator(_field, filters);
				if (labelDescription) {
					filterDescription[field.name] = labelDescription;
				}
			});
		}
	};

	var openCreateNewFilterWidget = function(popupWidget) {
		var createNewFilterWidget = filterWidgetsCreator.createNewFilterWidget(gridWidget, functionsCreateNewFilterWidget);
		createNewFilterWidget.getField('type').dataSource.data = zhCustomizationsService.getCustomizationsRepositoriesTypes();
		createNewFilterWidget.newRow();
		ScreenService.openPopup(createNewFilterWidget);
	};

	var createNewFilter = function(widget) {
		if (widget.isValid()) {
			var filterName = widget.currentRow.name;
			var customizationRepository = widget.currentRow.type;
			var preferenceHeader = createFilterHeader(filterName, customizationRepository);

			ScreenService.closePopup();
			templateManager.updateTemplate();
			var preferencesListWidget = gridWidget.__filterConfig;

			self.preferencesListOnEnter(preferencesListWidget).then(function() {
				openEditFilter(preferencesListWidget, preferenceHeader);
			});
		}
	};

	var createFilterHeader = function(filterName, customizationRepository) {
		var header = {
			"label": filterName,
			"widgetId": gridWidget.id,
			"containerId": gridWidget.container.id,
			"customizationRepository": customizationRepository,
			"unsync_id": new Date().getTime(),
			"customizationType": "filter",
			"mapUserIdDefault": {
				"default": false
			},
			"defaultPreference": false
		};
		return header;
	};

	var deletePreferenceClick = function(widget) {
		return zhCustomizationsService.removeCustomizations(widget.getCheckedRows()).then(function() {
			self.preferencesListOnEnter(widget).then(function() {
				templateManager.updateTemplate();
			});
		});
	};

	var openPopupSelectDestinyRepository = function(filterListWidget) {
		var traferFilterTypeWidget = filterWidgetsCreator.createChangeFilterTypeWidget(gridWidget, functionsTranferFilterType);
		traferFilterTypeWidget.newRow();
		traferFilterTypeWidget.getField('filterType').dataSource.data = zhCustomizationsService.getCustomizationsRepositoriesTypes();
		var toTransferLayouts = filterListWidget.getCheckedRows();
		var isSomeLayoutConnectedToAnVisuzalization = false;
		var promises = toTransferLayouts.map(function(filter) {
			return zhCustomizationsService.getCustomizationView(gridWidget, filter.customizationRepository, filter.customizationType, filter._id).then(function(connectedView) {
				if (connectedView) {
					isSomeLayoutConnectedToAnVisuzalization = true;
				}
			});
		});
		ZHPromise.all(promises).then(function() {
			if (!isSomeLayoutConnectedToAnVisuzalization) {
				ScreenService.openPopup(traferFilterTypeWidget);
			} else {
				ScreenService.showMessage(ScreenService.i18n("Filter connected with a customizations. It can't be transferred."));
			}
		});
	};

	var saveTransferFilterClick = function(widgetChangePreferenceType) {
		var newFilterType = widgetChangePreferenceType.getField("filterType").value();
		zhCustomizationsService.transferCustomizationsRepository(gridWidget, gridWidget.__filterConfig.getCheckedRows(), newFilterType).then(function() {
			fillFilterList(gridWidget.__filterConfig);
			ScreenService.closePopup();
		});
	};

	var openEditFilter = function(filterListWidget, row) {
		var filterFormWidget = filterWidgetsCreator.createFormFilterWidget(gridWidget, functionsFormFilterWidget);
		filterFormWidget.showAsModal = true;
		filterFormWidget.label = ScreenService.i18n("Filter") + " - " + row.label;
		filterFormWidget.filterSelected = row;
		filterFormWidget.fields = prepareFieldstoFormFilter(factoryFields(filterFormWidget));
		var searchFields = gridWidget.hideSearchFieldsOnFilter ? [] : prepareFieldstoFormFilter(factorySearchFields(filterFormWidget));
		filterFormWidget.fields = filterFormWidget.fields.concat(searchFields);
		filterFormWidget.currentRow = buildCurrentRowByFilters(row.filters, gridWidget, row._filterRow);
		ScreenService.openPopup(filterFormWidget);
	};

	var removeConflictingProperties = function(field) {
		var conflictingProperties = ['selectWidget'];
		conflictingProperties.forEach(function(property) {
			delete field[property];
		});
	};

	var factoryFields = function(filterForm) {
		return gridWidget.fields.filter(function(field) {
			return !Util.isUndefined(field.template);
		}).map(function(field) {
			var factoredField = metaDataFactory.fieldFactory(field);
			removeConflictingProperties(factoredField);
			factoredField.widget = filterForm;
			return factoredField;
		});
	};

	var factorySearchFields = function(filterForm) {
		var fields = [];
		fields.push(metaDataFactory.fieldFactory({
			"id": "SEARCHLIST__ALL",
			"label": ScreenService.i18n("All Fields"),
			"name": "SEARCHLIST__ALL",
			"class": 12,
			"template": "field/text-edit.html",
			"isVisible": true,
			"showOnList": true,
			"descending": true,
			"fieldGroup": "COLUMNS_FILTER"
		}, filterForm));

		fields = fields.concat(gridWidget.fields.filter(function(field) {
			return !field.skipSearch && Util.isUndefined(field.template) && !filterForm.getField(field.name);
		}).map(function(field) {
			var factoredField = metaDataFactory.fieldFactory(field);
			delete factoredField.$$hashKey;
			factoredField.id = "SEARCHLIST_"+factoredField.id;
			factoredField.name = "SEARCHLIST_"+factoredField.name;
			factoredField.class = 6;
			factoredField.isVisible = true;
			factoredField.readOnly = false;
			factoredField.validations = {};
			factoredField.template = "field/text-edit.html";
			factoredField.fieldGroup = "COLUMNS_FILTER";
			factoredField.widget = filterForm;
			return factoredField;
		}));

		return fields;
	};

	var prepareFieldstoFormFilter = this.prepareFields = function(fields) {
		return fields.map(function(field) {
			return field.filterProperties ? mergeFieldNewProperties(field, field.filterProperties) : field;
		});
	};

	var saveAndApplyFilter = function(filterFormWidget) {
		if (filterFormWidget.isValid()) {
			saveFilter(filterFormWidget).then(function() {
				self.applyFilter(filterFormWidget.filterSelected, gridWidget, function() {
					ScreenService.closePopup();
				});
			});
		}
	};

	var saveFilter = function(filterFormWidget) {
		filterFormWidget.filterSelected._filterRow = ZhUtil.clearRow(filterFormWidget.currentRow);
		filterFormWidget.filterSelected.filters = ZhFilterBuilder.buildFilter(filterFormWidget.fields.slice(), filterFormWidget.currentRow);
		removeNotInterestingProperties(filterFormWidget.filterSelected);
		return zhCustomizationsService.saveCustomization(filterFormWidget.filterSelected, 'filter').then(function() {
			self.preferencesListOnEnter(gridWidget.__filterConfig).then(function() {
				ScreenService.closePopup();
				templateManager.updateTemplate();
			});
		});
	};

	var removeNotInterestingProperties = function(object) {
		Object.keys(object).forEach(function(key) {
			if (~key.indexOf('__')) {
				delete object[key];
			}
		});
	};

	var applyFilterActionOnClick = function(filter) {
		self.applyFilter(filter, gridWidget);
		ScreenService.closePopup(true);
		gridWidget.activate();
	};

	var mergeFieldNewProperties = function(field, filterProperties) {
		var keys = Object.keys(field);
		keys = keys.concat(
			Object.keys(filterProperties).filter(function(key) {
				return keys.indexOf(key) == -1;
			})
		);
		keys.forEach(function(property) {
			field[property] = filterProperties.hasOwnProperty(property) ? filterProperties[property] : field[property];
		});
		if (field.dataSource && !(field.dataSource instanceof DataSource)) {
			field.dataSource = metaDataFactory.factoryDataSource(field.dataSource, field.dataSourceFilter, field);
		}
		return field;
	};

	var buildCurrentRowByFilters = function(filters, gridWidget, descriptionValues) {
		filters = Util.clone(filters) || [];
		var builtCurrentRow = descriptionValues || {};

		if (gridWidget && gridWidget.fields) {
			gridWidget.fields.forEach(function(field) {
				var fieldFilter = getFieldFilter(field.name, filters);
				if (fieldFilter) {
					if (fieldFilter.operator === 'LIKE' || fieldFilter.operator === 'LIKE_I') {
						if (fieldFilter.value[fieldFilter.value.length - 1] == '%') {
							fieldFilter.value = fieldFilter.value.substring(0, (fieldFilter.value.length - 1));
						}
						if (fieldFilter.value[0] == '%') {
							fieldFilter.value = fieldFilter.value.substring(1, fieldFilter.value.length);
						}
					} else if (fieldFilter.operator && ~fieldFilter.operator.indexOf("BETWEEN")) {
						fieldFilter.value = {
							start: fieldFilter.value[0],
							end: fieldFilter.value[1],
						};
					}
					builtCurrentRow[field.name] = fieldFilter.value;
					/*
					 dumb reload, should remove this in the next release!
					*/
					field.reload();
				}
			});
		}

		// add SearchList values
		filters.filter(function(filter) {
			return ~filter.name.indexOf("SEARCHLIST_");
		}).forEach(function(filter) {
			var filterValue = filter.value;
			if (filterValue[filterValue.length - 1] == '%') {
				filterValue = filterValue.substring(0, (filterValue.length - 1));
			}
			if (filterValue[0] == "%") {
				filterValue = filterValue.substring(1, filterValue.length);
			}
			builtCurrentRow[filter.name] = filterValue;
		});
		return builtCurrentRow;
	};

	this.inArray = function(elem, array) {
		return -1 != (Util.isArray(array) ? array.indexOf(elem) : -1);
	};

	var getFieldFilter = function(fieldName, filters) {
		return filters.filter(function(filter) {
			return filter.name == fieldName;
		}).shift();
	};

	var closeFilterPopup = function() {
		ScreenService.closePopup();
		gridWidget.activate();
		if (gridWidget.afterGridFilterCancel) {
			gridWidget.afterGridFilterCancel();
		}
	};

	this.updateFilterInfo = function(gridWidget, filters) {
		var stripeFilterFields = [];
		var currentFilter = buildCurrentRowByFilters(filters, gridWidget);
		var filterLabel = "";
		gridWidget._filterInfo = filterLabel;

		if (gridWidget.customizationsApplied && gridWidget.customizationsApplied.filter && gridWidget.customizationsApplied.filter != "edited" && gridWidget.customizationsApplied.filter != "default") {
			zhCustomizationsService.getWidgetCustomizations(gridWidget, "filter").then(function(filters) {
				filters.filter(function(filter) {
					if (filter._id == gridWidget.customizationsApplied.filter) {
						filterLabel = "<b>" + ScreenService.i18n("Filter") + ":</b> " + filter.label;
						gridWidget._filterInfo = filterLabel;
						ScreenService.updateTemplate();
					}
				});
			});
		} else {
			filterLabel = [];
			if (Util.isArray(filters)) {
				filters.forEach(function(filter) {
					var field = gridWidget.getField ? gridWidget.getField(filter.name) : undefined;
					if (field) {
						if (hasSelectTemplate(field)) {
							currentFilter[filter.name] = getSelectFieldDescriptionValue(field, currentFilter[filter.name]);
						}
						var fieldLabel = ScreenService.i18n(field.label);
						var nextField;
						if (filterDescription[field.name]) {
							nextField = "<b>" + fieldLabel + ":</b> " + filterDescription[field.name];
						} else if (filter.operator == "BETWEEN") {
							nextField = "<b>" + fieldLabel + ":</b> " + currentFilter[filter.name].start + ' - ' + currentFilter[filter.name].end;
						} else {
							nextField = "<b>" + fieldLabel + ":</b> " + currentFilter[filter.name];
						}

						filterLabel.push(nextField);
					} else if (~filter.name.indexOf("SEARCHLIST_")) {
						if (filter.name == "SEARCHLIST__ALL") {
							filterLabel.push("<b>" + ScreenService.i18n("All Fields") + ":</b> " + currentFilter[filter.name]);
						} else {
							field = gridWidget.getField ? gridWidget.getField(filter.name.substring(11)) : undefined;
							if (field) {
								if (hasSelectTemplate(field)) {
									currentFilter[filter.name] = getSelectFieldDescriptionValue(field, currentFilter[filter.name]);
								}
								filterLabel.push("<b>" + ScreenService.i18n(field.label) + ":</b> " + currentFilter[filter.name]);
							}
						}
					}
				});
			} else if (Util.isObject(filters)) {
				Object.keys(filters).forEach(function(key) {
					if (key == "_ALL") {
						filterLabel.push("<b>" + ScreenService.i18n("All Fields") + ":</b> " + filters[filter.name]);
					} else {
						field = gridWidget.getField(key);
						if (field) {
							if (field.template && hasSelectTemplate(field)) {
								filters[key] = getSelectFieldDescriptionValue(field, filters[key]);
							}
							filterLabel.push("<b>" + ScreenService.i18n(field.label) + ":</b> " + filters[key]);
						}
					}
				});
			}

			filterLabel = filterLabel.join("<br />");
			gridWidget._filterInfo = filterLabel;
			ScreenService.updateTemplate();
		}
	};

	var hasSelectTemplate = function(field) {
		return field.template && ~field.template.indexOf('select-native.html');
	};

	var getSelectFieldDescriptionValue = function(field, value) {
		if (field.dataSource && field.dataSource.data) {
			var selectedRow = field.dataSource.data.filter(function(row) {
				return value == row[field.valueField].toString();
			});
			if (selectedRow.length > 0) {
				value = selectedRow[0][field.descriptionField];
			}
		}
		return value;
	};

	var factoryStripeFields = function(stripe, field) {
		factoredField = metaDataFactory.fieldFactory(field);
		factoredField.widget = stripe;
		if (stripe.__originalFields) {
			stripe.fields = stripe.__originalFields.concat([factoredField]);
		} else {
			stripe.fields = factoredField;
		}
	};

	var defaultFilterProperties = {};

	this.setFilterProperties = function(gridWidget, currentRow) {
		var filter = {
			"_id": "default",
			filters: ZhFilterBuilder.buildFilter(gridWidget.fields, currentRow)
		};
		self.applyFilter(filter, gridWidget);
	};

	this.setDefaultFilterProperties = function(widget, currentRow) {
		gridWidget = widget;
		defaultFilterProperties[gridWidget.id] = ZhFilterBuilder.buildFilter(gridWidget.fields, currentRow);
		if (!gridWidget.filterApplied || !gridWidget.filterApplied.length) {
			self.setFilterProperties(gridWidget, currentRow);
		}
	};

	this.getDefaultFilterProperties = function(gridWidget) {
		return defaultFilterProperties[gridWidget.id];
	};

	this.getStaticDataSourceFilters = function(dataSourceFilter) {
		return dataSourceFilter.filter(function(filter) {
			return !filter.static;
		});
	};

	var setAsDefaultPreference = function(row, widget) {
		var toUpdate = [];
		var userRegistered = preferencesUserService.getPreferenceUserRegistered() || "default";
		var preferenceSetAsDefault = widget.dataSource.data.filter(function(preference) {
			return preference.mapUserIdDefault[userRegistered] === true && preference._id !== row._id;
		}).shift();
		if (preferenceSetAsDefault) {
			preferenceSetAsDefault.mapUserIdDefault[userRegistered] = false;
			removeNotInterestingProperties(preferenceSetAsDefault);
			toUpdate.push(preferenceSetAsDefault);
		}
		removeNotInterestingProperties(row);
		row.mapUserIdDefault[userRegistered] = !row.mapUserIdDefault[userRegistered];
		toUpdate.push(row);
		zhCustomizationsService.saveCustomization(toUpdate, 'filter').then(function() {
			self.preferencesListOnEnter(widget).then(function() {
				ScreenService.hideLoader();
				ScreenService.updateTemplate();
			});
		});
	};

	var functionsFilterListWidget = {
		"fillFilterList": fillFilterList,
		"openEditFilter": openEditFilter,
		"openCreateNewFilterWidget": openCreateNewFilterWidget,
		"deletePreferenceClick": deletePreferenceClick,
		"openPopupSelectDestinyRepository": openPopupSelectDestinyRepository,
		"applyFilterActionOnClick": applyFilterActionOnClick,
		"closeFilterPopup": closeFilterPopup,
		"setAsDefaultPreference": setAsDefaultPreference,
		"restoreDefaultFilter": self.restoreDefaultFilter,
		"preferencesListOnEnter": self.preferencesListOnEnter
	};

	var functionsCreateNewFilterWidget = {
		"createNewFilter": createNewFilter
	};

	var functionsTranferFilterType = {
		"saveTransferFilterClick": saveTransferFilterClick
	};

	var functionsFormFilterWidget = {
		"saveAndApplyFilter": saveAndApplyFilter,
		"saveFilter": saveFilter
	};

	var functionsFormCurrentFilterWidget = {
		"currentFilterClickPopup": currentFilterClickPopup
	};

}

Configuration(function(ContextRegister) {
	var zhFilterPreferences;
	ContextRegister.register("zhFilterPreferences", function(ScreenService, templateManager, ZhFilterBuilder, ZhUtil, RepositoryFactory, Query, eventEngine, ZHPromise, ZhLogger, ApplicationContext, zhCustomizationsService, filterWidgetsCreator, eventAggregator, preferencesUserService, DecodeValueFieldService) {
		zhFilterPreferences = new ZhFilterPreferences(ScreenService, templateManager, ZhFilterBuilder, ZhUtil, RepositoryFactory, Query, eventEngine, ZHPromise, ZhLogger, ApplicationContext, zhCustomizationsService, filterWidgetsCreator, eventAggregator, preferencesUserService, DecodeValueFieldService);
		return zhFilterPreferences;
	});
	ContextRegister.register("ZhGridFilter", zhFilterPreferences);
});